#include "dsp.h"
#include "lcd.h"
#include <avr/io.h>
#include <util/delay.h>

// create an array for the delay
char array[1000]; 

//define variables
int j;
int fx;
int mode;
int value50;
int value300;
int value10000;
int delayed;

void setup() {
    setupIO();
    
    //set initial values
    j = 50;
    value50 = 50;
    value300 = 300;
    value10000 = 1000;
    delayed = 0;
}

unsigned int analogRead(int channel) {
    ADCSRA |= (1 << ADSC);         //start conversion
    loop_until_bit_is_set(ADCSRA, ADIF);  //wait until conversion is done: until ADIF from ADCSRA is set to 1
    unsigned int value = ADC;
    return value;
}

void readKnobs() {
    //read the rotary switch
    //and determine which effect is selected
    //dividing by 75 ensures proper discrete values
    //for if statements above
    mode = analogRead(2);
    mode = mode / 75;
        
    //reads the effects pot to adjust
    //the intensity of the effects 
    fx = analogRead(3);

}

void loop() {
  
    //check status of the effect potentiometer and rotary switch
    readKnobs();
    int foundMode = 0;
    LCD_writeInstr(LCD_INSTR_clearDisplay);
  
    if (mode == 6) {
        value300 = 1 + ((float) fx / (float) 3);        
        if (delayed > value300) {  
            char input = analogRead(left);
            input = (input >> 6 << 6);
            output(left, input);
            delayed = 0;
        } 
        delayed++;
        foundMode = 1;
        LCD_print("Selected Mode: Bitcrush");
    }
    

    if (mode == 7) {
        value50 = 1 + ((float) fx / (float) 20);    
        char input = analogRead(left);
        input = (input * value50); 
        output(left, input);
        foundMode = 1;
        LCD_print("Selected Mode: Overdrive");
    }


    if (mode == 9) {
        for (int i = 0; i < 1000; i ++) {
            output(left, array[i]);
            array[i] = analogRead(left);
        }
        foundMode = 1;
        LCD_print("Selected Mode: Delay");
    }


    if(mode == 10) {
        value10000 = fx * 10;  
        if (delayed > value10000) { 
            for(int i = 0; i < 1000; i ++) { // set up a loop    
                array[i] = array[i] + array[i - 1]; //removes noise and delay
                output(left, array[i]);
                array[i] = analogRead(left);
            }
            delayed = 0;
        }
        delayed++;
        foundMode = 1;
        LCD_print("Selected Mode: Clean");
    }
    
    if (mode == 11) {
        value50 = 1 + ((float) fx / (float) 20);    
        char input = analogRead(left);
        input = (input * j);
        output(left, input);
        
        j = j - 1;
        if (j <= 0) {
            j = value50;
        }
        foundMode = 1;
        LCD_print("Selected Mode: Ring Modulator");
    }

    if (!foundMode) {
        LCD_print("Pedal is running");
    }
    
}

int main() {
    setupIO();
    LCD_init();
    LCD_print("Pedal is running");

    while (1) {
        loop();
    }
}
